
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007005
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib ) );
__DATA__
ACKNOWLEDGEMENTS
AddBulkDomains
Addon
AnnoCPAN
Auth
Autoverification
AuxBilling
Availability
BUILDARGS
C11
C12
C21
C31
C32
CheckNSStatus
Command
Compatable
Contact
Cozens
DeleteNameServer
DisableServices
Domain
DomainAvailability
DomainRequest
DomainTransfer
EnableServices
ErrX
FQDN
FQDNs
FirstName
GetContacts
GetDNS
GetDomainInfo
GetRegLock
GetRenew
GetSubAccountPassword
GetWPPSInfo
GetWhoisContact
ICANN
IRTP
IRTPDetail
ModifyNS
NameSpinner
Net
OrderID
P1
P2
P3
P4
P5
ParseDomain
PhoneNumber
PositiveInt
PrivateNameServer
PurchaseServices
RRPText
Raw
RegisterNameServer
Registration
RenewServices
Robert
Role
SLD
SLDs
Service
SetRegLock
SetRenew
Simões
Stone
Str
TLD
TLDs
TechFirstName
Transfer
Types
UKFSN
UpdateNameServer
WHOIS
WPPS
WWW
addon
address1
address2
assoicated
bona
cartesian
drzigman
eNom
enom
falsey
fide
functionalty
gTLD
google
html
ip
irtpsettings
lib
marshallers
nameserver
nameservers
ns
ns1
params
reseller
resellers
sld
slds
tld
tlds
truthy
tv
uk
v0
v1
whois
xml
zipcode
