
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    print qq{1..0 # SKIP these tests are for testing by the author\n};
    exit
  }
}

use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.19

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Net/eNom.pm',
    'lib/WWW/eNom.pm',
    'lib/WWW/eNom/Contact.pm',
    'lib/WWW/eNom/Domain.pm',
    'lib/WWW/eNom/DomainAvailability.pm',
    'lib/WWW/eNom/DomainRequest/Registration.pm',
    'lib/WWW/eNom/DomainRequest/Transfer.pm',
    'lib/WWW/eNom/DomainTransfer.pm',
    'lib/WWW/eNom/IRTPDetail.pm',
    'lib/WWW/eNom/PhoneNumber.pm',
    'lib/WWW/eNom/PrivateNameServer.pm',
    'lib/WWW/eNom/Role/Command.pm',
    'lib/WWW/eNom/Role/Command/Contact.pm',
    'lib/WWW/eNom/Role/Command/Domain.pm',
    'lib/WWW/eNom/Role/Command/Domain/Availability.pm',
    'lib/WWW/eNom/Role/Command/Domain/PrivateNameServer.pm',
    'lib/WWW/eNom/Role/Command/Domain/Registration.pm',
    'lib/WWW/eNom/Role/Command/Domain/Transfer.pm',
    'lib/WWW/eNom/Role/Command/Raw.pm',
    'lib/WWW/eNom/Role/Command/Service.pm',
    'lib/WWW/eNom/Role/ParseDomain.pm',
    'lib/WWW/eNom/Types.pm',
    't/00-compile.t',
    't/Preload.pm',
    't/author-critic.t',
    't/author-eol.t',
    't/author-mojibake.t',
    't/author-no-tabs.t',
    't/author-pod-coverage.t',
    't/author-pod-linkcheck.t',
    't/author-pod-spell.t',
    't/author-pod-syntax.t',
    't/author-portability.t',
    't/author-test-version.t',
    't/contact/00-object.t',
    't/contact/01-organization_name_optional_fields.t',
    't/domain/00-object.t',
    't/domainavailability/00-object.t',
    't/domainrequest/registration/00-object.t',
    't/domainrequest/transfer/00-object.t',
    't/domainrequest/transfer/01-required_contacts.t',
    't/domaintransfer/00-object.t',
    't/eNom/00-object.t',
    't/irtp_detail/00-object.t',
    't/legacy/01-check.t',
    't/legacy/02-response_type.t',
    't/legacy/03-net-enom.t',
    't/lib/Test/WWW/eNom.pm',
    't/lib/Test/WWW/eNom/Contact.pm',
    't/lib/Test/WWW/eNom/Domain.pm',
    't/lib/Test/WWW/eNom/Domain/Transfer.pm',
    't/lib/Test/WWW/eNom/Service.pm',
    't/phonenumber/00-object.t',
    't/phonenumber/01-new.t',
    't/private_name_server/00-object.t',
    't/release-cpan-changes.t',
    't/release-dist-manifest.t',
    't/release-distmeta.t',
    't/release-kwalitee.t',
    't/release-meta-json.t',
    't/release-minimum-version.t',
    't/release-unused-vars.t',
    't/role/command/00-object.t',
    't/role/command/contact/00-object.t',
    't/role/command/contact/01-get_contacts_by_domain_name.t',
    't/role/command/contact/02-update_contacts_for_domain_name.t',
    't/role/command/domain/00-object.t',
    't/role/command/domain/01-get_is_domain_locked_by_name.t',
    't/role/command/domain/02-get_domain_name_servers_by_name.t',
    't/role/command/domain/03-get_is_domain_auto_renew_by_name.t',
    't/role/command/domain/04-get_domain_created_date_by_name.t',
    't/role/command/domain/05-get_domain_by_name.t',
    't/role/command/domain/06-enable_domain_lock_by_name.t',
    't/role/command/domain/07-disable_domain_lock_by_name.t',
    't/role/command/domain/08-enable_domain_auto_renew_by_name.t',
    't/role/command/domain/09-disable_domain_auto_renew_by_name.t',
    't/role/command/domain/10-update_nameservers_for_domain_name.t',
    't/role/command/domain/11-renew_domain.t',
    't/role/command/domain/12-email_epp_key_by_name.t',
    't/role/command/domain/availability/00-object.t',
    't/role/command/domain/availability/01-check_availability.t',
    't/role/command/domain/availability/02-suggest_domain_names.t',
    't/role/command/domain/private_name_server/00-object.t',
    't/role/command/domain/private_name_server/01-create_private_nameserver.t',
    't/role/command/domain/private_name_server/02-retrieve_private_nameserver_by_name.t',
    't/role/command/domain/private_name_server/03-delete_private_nameserver.t',
    't/role/command/domain/private_name_server/04-update_private_nameserver_ip.t',
    't/role/command/domain/registration/00-object.t',
    't/role/command/domain/registration/01-register_domain.t',
    't/role/command/domain/registration/02-register_domain-us.t',
    't/role/command/domain/transfer/00-object.t',
    't/role/command/domain/transfer/01-transfer_domain.t',
    't/role/command/domain/transfer/02-get_transfer_by_order_id.t',
    't/role/command/domain/transfer/03-get_transfer_by_name.t',
    't/role/command/domain/transfer/04-get_transfer_order_id.t',
    't/role/command/raw/00-object.t',
    't/role/command/service/00-object.t',
    't/role/command/service/01-purchase_domain_privacy_for_domain.t',
    't/role/command/service/02-get_is_privacy_auto_renew_by_name.t',
    't/role/command/service/03-get_privacy_expiration_date_by_name.t',
    't/role/command/service/04-enable_privacy_auto_renew_for_domain.t',
    't/role/command/service/05-disable_privacy_auto_renew.t',
    't/role/command/service/06-get_is_privacy_purchased_by_name.t',
    't/role/command/service/07-enable_privacy_by_name.t',
    't/role/command/service/08-disable_privacy_by_name.t',
    't/role/command/service/09-renew_privacy.t',
    't/role/command/service/10-get_domain_privacy_wholesale_price.t',
    't/role/parsedomain/00-object.t',
    't/role/parsedomain/01-domain_parts.t'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
